﻿using PlcSiemens.Core.Extension;
using ServiceCenter;
using WCS.Core;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.Protocol.BCR;
using WCS.WorkEngineering.Protocol.RGV;
using WCS.WorkEngineering.Protocol.SRM;
using WCS.WorkEngineering.Protocol.Station;
using WCS.WorkEngineering.Protocol.Truss;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;

namespace WCS.WorkEngineering
{
    /// <summary>
    /// 业务工程配置信息
    /// </summary>
    public static class WorkStart
    {
        /// <summary>
        ///  初始化 设备信息
        /// </summary>

        public static void InitializeDeviceInfo()
        {
            #region 设置设备IP集合

            ServiceHub.DeviceIPList = new List<string>() { };

            #endregion 设置设备IP集合

            #region 初始化输送机相关信息

            #region 基本信息

            List<StationSegmentInfo> segmentInfo = new List<StationSegmentInfo>
            {
            };

            foreach (var item in segmentInfo)
            {
                for (int i = item.Start; i <= item.End; i++)
                {
                    var conv = new Device(i.ToString());
                    conv.AddFlag(DeviceFlags.输送机);
                    conv.AddProtocol<IStation520>((i - item.Start) * 14, 520, item.IP);
                    conv.AddProtocol<IStation521>((i - item.Start) * 16, 521, item.IP);
                    conv.AddProtocol<IStation523>((i - item.Start) * 12, 523, item.IP);
                }
            }

            #endregion 基本信息

            #region 扫码器

            List<BcrInfo> bcrInfo = new List<BcrInfo>
            {
            };

            foreach (var item in bcrInfo)
            {
                for (int i = 0; i < item.DeviceNo.Length; i++)
                {
                    var conv = new Device("BCR" + item.DeviceNo[i]);
                    conv.AddFlag(DeviceFlags.扫码);
                    int pos = i * 20;
                    conv.AddProtocol<IBCR81>(pos, 81, item.IP);
                }
            }

            #endregion 扫码器

            #region 外检信息

            List<ShapeInfo> shapeInfo = new List<ShapeInfo>
            {
            };

            foreach (var item in shapeInfo)
            {
                for (int i = 0; i < item.DeviceNo.Length; i++)
                {
                    var conv = Device.All.FirstOrDefault(x => x.Code == item.DeviceNo[i].ToString());
                    conv.AddFlag(DeviceFlags.外检);
                    int pos = i * 14;
                    conv.AddProtocol<IStation91>(pos, 91, item.IP);
                }
            }

            #endregion 外检信息

            #endregion 初始化输送机相关信息

            #region 初始化RGV相关信息

            List<RGVSegmentInfo> RgvInfo = new List<RGVSegmentInfo>
            {
            };

            foreach (var item in RgvInfo)
            {
                var conv = new Device($"RGV{item.Code}");
                conv.AddFlag(DeviceFlags.RGV);
                conv.AddProtocol<IRGV520>(0, 520, item.IP);
                conv.AddProtocol<IRGV521>(0, 521, item.IP);
                //conv.AddProtocol<IStation523>(, 523, item.IP);
            }

            #endregion 初始化RGV相关信息

            #region 初始化桁架相关信息

            List<TrussSegmentInfo> TrussInfo = new List<TrussSegmentInfo>
            {
            };

            foreach (var item in TrussInfo)
            {
                var conv = new Device($"Truss{item.Code}");
                conv.AddFlag(DeviceFlags.桁架);
                conv.AddProtocol<ITruss520>(0, 520, item.IP);
                conv.AddProtocol<ITruss521>(0, 521, item.IP);
                conv.AddProtocol<ITruss523>(0, 523, item.IP);
                conv.AddProtocol<ITruss530>(0, 530, item.IP);
                conv.AddProtocol<ITruss531>(0, 531, item.IP);
            }

            #endregion 初始化桁架相关信息

            #region 初始化堆垛机相关信息

            int ip = 41;

            for (int i = 0; i <= 1; i++)
            {
                var srm = new Device($"SRM{i + 1}");
                srm.AddFlag(DeviceFlags.堆垛机);
                ip = i == 0 ? ip : ip + 8;
                //三台堆垛机IP主机位分别是 21 31 41
                srm.AddProtocol<ISRM520>(0, 520, $"10.30.37.{ip}");
                srm.AddProtocol<ISRM521>(0, 521, $"10.30.37.{ip}");
                srm.AddProtocol<ISRM537>(0, 537, $"10.30.37.{ip}");

                //增加巷道
                var tunnel = new Device($"TY{i + 1}");
                tunnel.AddFlag(DeviceFlags.巷道);
            }

            #endregion 初始化堆垛机相关信息

            #region 配置路径信息

            List<RouteInfo> routeInfos = new List<RouteInfo>
            {
            };

            foreach (var item in routeInfos)
            {
                var device = Device.All.FirstOrDefault(x => x.Code == item.DeviceCode);
                device.AddTarget(item.NextList);

                foreach (var next in item.NextList)
                {
                    var dev = Device.All.FirstOrDefault(x => x.Code == next);
                    dev.Parent = device;
                }
            }

            #endregion 配置路径信息

            #region 标签配置

            Dictionary<DeviceFlags, List<string>> devices = new Dictionary<DeviceFlags, List<string>>
            {
            };

            devices.ForEach(item =>
            {
                item.Value.ForEach(code =>
                {
                    var device = Device.All.FirstOrDefault(v => v.Code == code);
                    device.AddFlag(item.Key);
                });
            });

            #endregion 标签配置
        }
    }
}
